/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ComponentsView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.TitlePane");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");

var replaceNewLineWithPaddedBreaks = com.ibm.team.repository.web.ui.internal.admin.util.replaceNewLineWithPaddedBreaks;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var STATUS_OK_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/images/verify_obj.gif");
var STATUS_ERR_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/status-error.gif");

var ALIAS_MAP = {};
var ALIAS_LABEL = " (Alias: "

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ComponentsView", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ComponentsView.html"),

	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.ComponentsView.superclass.postCreate.apply(this);
		this._fetchAndDisplayComponentsInfo();
	},
	
	_fetchAndDisplayComponentsInfo: function() {
		var srh= new ServiceResponseHandler(this, "_displayComponentsInfo", "_handleComponentsFetchError");
		ServerStatusClient.getComponents(srh)
	},

	_fetchAndUpdateServiceStatus: function(services, detailsDiv, serviceListDiv) {
		var serviceUpdater = {
			_handleStatus: function(statusCollection) {
				serviceListDiv.appendChild(document.createElement("div"));
				//dojox.data.dom.removeChildren(serviceListDiv);
				if(statusCollection) {
					// sort by name
					statusCollection.sort(
							function(a, b) {
								return a.interfaceName.toLowerCase() > b.interfaceName.toLowerCase() ? 1 : -1;
					});
				
					for (var i=0; i<statusCollection.length; i++) {
						var original = this._findDivByServiceName(statusCollection[i].interfaceName, serviceListDiv);
						if (original == null) {
							return; //early return
						} 
						if (statusCollection[i].code === "OK") {
							dojo.addClass(original.parentNode, "status-ok");
							var okImage = this._createOkServiceStatusSection(statusCollection[i]);
							original.parentNode.insertBefore(okImage, original);
						} else {
							var hasError = true;
							var serviceDom = document.createElement("div");
							this._appendErrorServiceStatusSection(statusCollection[i], serviceDom);
							original.parentNode.replaceChild(serviceDom, original);
						}
					}
					dojo.removeClass(serviceListDiv,"hide-contents");
				}
				detailsDiv.appendChild(serviceListDiv);
				if (hasError)
					this._displayComponentStatusError(serviceListDiv, true);
				else
					this._displayComponentStatusError(serviceListDiv, false);
			},
			
			_findDivByServiceName: function (name, container) {
				var nodes = container.childNodes;
				for (var i = 0; i < nodes.length; i++) {
					if (nodes[i].firstChild.firstChild != null && nodes[i].firstChild.firstChild.data != null) {
						if (0 === nodes[i].firstChild.firstChild.data.indexOf(name)) {
							return nodes[i].firstChild;
						}
					}
				}
				return null;
			},
			
			_displayComponentStatusError: function (serviceDiv, isError){
				var displayStyle = "none";
				if (isError) {
					displayStyle = "inline";
				} 
				//KMB TODO: update to less fragile way to get to component label
				dojo.query(".component-status",serviceListDiv.parentNode.parentNode.firstChild)[0].style.display = displayStyle;
			},
			
			_createOkServiceStatusSection: function(status) {
				var statusSpan = document.createElement("img");
				statusSpan.src = STATUS_OK_IMG_SRC;
				statusSpan.className = "service-status";
				statusSpan.title = status.message;
				return statusSpan;
			},
			
			_appendErrorServiceStatusSection: function(status, serviceDiv) {
				var nameLabel = status.interfaceName;
				if (ALIAS_MAP[status.interfaceName] != null) {
					nameLabel = status.interfaceName + ALIAS_LABEL + ALIAS_MAP[status.interfaceName] + ")";
				}
				var statusDetails = new dijit.TitlePane({title: nameLabel});
				var errorContent = document.createElement("div");
				var errorSummary = document.createElement("div");
				errorSummary.className = "status-summary";
				errorSummary.innerHTML = status.message;
				errorContent.appendChild(errorSummary);
				var errorDetails = document.createElement("div");
				errorDetails.innerHTML = replaceNewLineWithPaddedBreaks(status.stacktrace, false);
				errorContent.appendChild(errorDetails);
				statusDetails.setContent(errorContent);
				
				statusDetails.toggle();
				statusDetails.arrowNode.title = status.message;
 				serviceDiv.appendChild(statusDetails.domNode);
			},
			
			_handleError: function(errorObj) {
				var serviceSpan = document.createElement("span");
				serviceSpan.innerHTML = "Error fetching server status";
				serviceListDiv.appendChild(serviceSpan);
			}
		};
		var srh= new ServiceResponseHandler(serviceUpdater, "_handleStatus", "_handleError");
		ServerStatusClient.postStatusForServiceList(srh, services);
	},

	_displayComponentsInfo: function(components) {
		dojox.data.dom.removeChildren(this.componentsArea);
		ALIAS_MAP = {};
	
		components.sort(
			function(a, b) {
				return a.id.toLowerCase() > b.id.toLowerCase() ? 1 : -1;
		}); 
		for (var i=0; i<components.length; i++) {
			var compDom = this._createComponentSection(components[i]);
			this.componentsArea.appendChild(compDom);
		}
	},
	
	_createComponentSection: function(component) {
		var div = document.createElement("div");
		div.className = "component com-ibm-team-repository-admin-table";
	
		var span = document.createElement("div");
		span.className = "heading";
		span.innerHTML = component.id;
		var img = document.createElement("img");
		img.className = "component-status";
		img.src = STATUS_ERR_IMG_SRC;
		span.insertBefore(img, span.firstChild);

		if (component.modelURI) {
			var model = document.createElement("div");
			model.className = "model";
			model.innerHTML = "<span>Model:</span> " + component.modelURI;
			span.insertBefore(model, span.firstChild);
		}		

		div.appendChild(span);
		var detailsDiv = document.createElement("div");
		detailsDiv.className = "component-details";
		div.appendChild(detailsDiv);
		
		var services = component.services;			
		if (services && services.length > 0) {
			var serviceList = document.createElement("div");
			serviceList.className = "service-list hide-contents";
			span = document.createElement("div");
			span.className = "service-list-header";
			span.innerHTML = "Services:";
			detailsDiv.appendChild(span);
									
			var statusRequest= {
				serviceNames: []
			};
			
			// sort by name
			services.sort(
				function(a, b) {
					return a.interfaceName.toLowerCase() > b.interfaceName.toLowerCase() ? 1 : -1;
				});

			for (var i=0;i<services.length;i++) {
				var serviceDiv = document.createElement("div");
				serviceDiv.className = "service";
				var serviceNameDiv = document.createElement("div");
				serviceNameDiv.className = "service-name";
				
				var nameLabel = services[i].interfaceName;
				if (services[i].aliasName != null) {
					nameLabel = this._formatAliasLabel(services[i]);
					ALIAS_MAP[services[i].interfaceName] = services[i].aliasName;
				} 
				serviceNameDiv.innerHTML = nameLabel;
				serviceDiv.appendChild(serviceNameDiv);
				
				statusRequest.serviceNames.push(services[i].interfaceName);
				serviceList.appendChild(serviceDiv);
			}
			setTimeout(dojo.hitch(this,this._fetchAndUpdateServiceStatus,statusRequest,detailsDiv,serviceList),0);
		}
		
		return div;	
	},
	
	_formatAliasLabel: function (service) {
		return service.interfaceName + "<span class=\"alias\">" + ALIAS_LABEL + service.aliasName + ") </span>";
	},
		
	_handleComponentsFetchError: function(errorObj) {
		this._displayErrorMessage("Error fetching component information", errorObj);
	},	

	_displayErrorMessage: function(message, errorObj) {
		if (errorObj) {
			message += " (detailed message: " + 
				(errorObj.status === 0 ? "server unreachable - check network connectivity or contact server admin" : errorObj.message) +
				").";
		}
		this._displayMessage(message, "errorMsg");
	},
	
	// do not call directly
	_displayMessage: function(message, className) {
		dojox.data.dom.removeChildren(this.messageArea);
		this.messageArea.className= className;
		this.messageArea.appendChild(document.createTextNode(message));
	}

});

})();