/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.ManageUserCALs");

dojo.require("dojox.data.dom");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var LicenseClient = com.ibm.team.repository.web.client.internal.licensing;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamContributorSelectionDialog = com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog;

function setText(e, text){
	e.appendChild(document.createTextNode(text));
}

var LICENSE_KEY_MANAGEMENT_ACTION = "#action=com.ibm.team.repository.admin.manageLicenses";
var SEARCH_ACTION = "action=com.ibm.team.repository.manageUserCALs";
var PAGE_SIZE = 50;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.ManageUserCALs", com.ibm.team.repository.web.ui.internal.admin.ItemList, {
	
	postCreate: function() {
		setText(this.heading, "Client Access License Management");
		setText(this.description, "Use this page to manage assignment of Client Access Licenses to users.");
		this.licenseMap = {};
		this.tableHeading.parentNode.removeChild(this.tableHeading);
		this.newItemButtonArea.parentNode.removeChild(this.newItemButtonArea);
		this.tabSection = document.createElement("div");
		this.tabSection.className = "tabSection";
		this.tableControls.appendChild(this.tabSection);
		this.licenseDescription = document.createElement("div");
		this.licenseDescription.className = "licenseDescription";
		this.tableControls.appendChild(this.licenseDescription);
		this.licenseCountArea = document.createElement("div");
		this.licenseCountArea.className = "licenseCountArea";
		this.tableControls.appendChild(this.licenseCountArea);
		this.controlButtonArea = document.createElement("div");
		this.tableControls.appendChild(this.controlButtonArea);
		this.assignButton = document.createElement("button");
		this.connect(this.assignButton,"onclick", function (e) {
			dojo.stopEvent(e);
			var userPicker = new TeamContributorSelectionDialog({ 'disableUsersWithLicense' : this.latestSearch.licenseName });
			this.connect(userPicker,"onAdd","assignLicenses");
		});
		this.assignButton.appendChild(document.createTextNode("Assign Client Access Licenses"));
		this.controlButtonArea.appendChild(this.assignButton);
		this.setHeaderRow(["Name", "User ID", "E-Mail"]);
		this.pageNumberContainer = document.createElement("div");
		this.pageNumberContainer.style.marginTop = "5px";
		this.pageNumberContainer.style.fontWeight = "normal";
		this.pageNumberContainer.style.display = "none";
		this.pageNumberContainer.appendChild(document.createTextNode("(Showing first " + PAGE_SIZE + " users)"));
		this.tableControls.appendChild(this.pageNumberContainer);
	},
	
	refresh: function () {
		this.clearState();
		var tr = document.createElement("tr");
		var td = document.createElement("td");
		td.appendChild(document.createTextNode("Searching for users..."));
		tr.appendChild(td);
		this.tbody.appendChild(tr);
		var srh = new ServiceResponseHandler(this, "getLicenseInfoCallback", "getLicenseInfoErrorCallback");
		LicenseClient.getLicenseInfo(srh);
	},
	
	show: function () {
		this.refresh();
	},
	
	reload: function () {
		this.refresh();
		this.runLatestSearch();
	},
	
	clearState: function () {
		dojox.data.dom.removeChildren(this.tbody);
		this.pageNumberContainer.style.display = "none";
		dojox.data.dom.removeChildren(this.messageBox);
		this.messageBox.className = "";
	},
	
	getLicenseInfoCallback: function (data) {
		dojox.data.dom.removeChildren(this.tabSection);
		if (data.serverLicense != null) {
			this.isTrialLicense = data.serverLicense.expirationTime ? true : false;
		}
		if (data.types != null) {
			var self = this;
			var licenses = LicenseClient.sortCALs(data.types);
			dojo.forEach(licenses, function (license) {
				if (self.defaultLicenseSearch == null) {
					self.defaultLicenseSearch = license.id;
				}
				self.licenseMap[license.id] = license;
				var link = document.createElement("a");
				link.licenseId = license.id;
				link.href = "#" + SEARCH_ACTION + "&licenseType=" + license.id;
				link.className = "tab";
				var topWrapper = document.createElement("div");
				topWrapper.className = "topWrapper";
				var topBorder = document.createElement("div");
				topBorder.className = "topBorder";
				topWrapper.appendChild(topBorder);
				var corners = document.createElement("div");
				corners.className = "corners";
				topWrapper.appendChild(corners);
				link.appendChild(topWrapper);
				var innerWrapper = document.createElement("div");
				innerWrapper.className = "innerWrapper";
				var title = document.createElement("div");
				title.className = "title";
				var licenseInfo = LicenseClient.getClientAccessLicenseInfo(data, license);
				title.appendChild(document.createTextNode(licenseInfo.name));
				innerWrapper.appendChild(title);
				link.appendChild(innerWrapper);
				self.tabSection.appendChild(link);
				self.licenseDescription[license.id] = license.description;
				if (self.licenseCountArea[license.id]) {
					self.licenseCountArea[license.id]["disabled"] = licenseInfo.available === -2;
					self.licenseCountArea[license.id]["disabledReason"] = licenseInfo.available === -2 ? license.disabledReason : "";
					self.licenseCountArea[license.id]["unlimited"] = licenseInfo.available === -1;
					self.licenseCountArea[license.id]["total"] = self.isTrialLicense ? license.max : license.total;
					if (self.latestSearch != null && self.latestSearch.licenseName == license.id) {
						self.showAvailableCount(licenseInfo.available);
					}
				}
				else {
					self.licenseCountArea[license.id] = {
						total: self.isTrialLicense ? license.max : license.total,
						unlimited: licenseInfo.available === -1,
						disabled: licenseInfo.available === -2,
						disabledReason: licenseInfo.available === -2 ? license.disabledReason : ""
					};
				}
				if (self.latestSearch == null) {
					self.runLicenseSearch(license.id, 0);
				}
				self.connect(link, "onclick", function (e) {
					e.target.blur();
				});
			});
			this.updateHeaderLinks();
			this.tableControls.style.display = "";
			this.tbody.parentNode.style.display = "";
		}
		else {
			this.tableControls.style.display = "none";
			this.tbody.parentNode.style.display = "none";
			var message = document.createElement("div");
			message.appendChild(document.createTextNode("There are no client access licenses installed in your repository.  You can go to the "));
			var link = document.createElement("a");
			link.href = LICENSE_KEY_MANAGEMENT_ACTION;
			link.appendChild(document.createTextNode("License Key Management"));
			message.appendChild(link);
			message.appendChild(document.createTextNode(" page to upload new Client Access Licenses."));
			this.displayPendingMessage(message, true);
		}
	},
	
	updateHeaderLinks: function () {
		if (this.tabSection.childNodes != null && this.latestSearch != null) {
			var activeLicense = this.latestSearch.licenseName;
			for (var i=0; i < this.tabSection.childNodes.length; i++) {
				if (this.tabSection.childNodes[i].licenseId == activeLicense) {
					dojo.toggleClass(this.tabSection.childNodes[i], "selected", true);
					this.tabSection.childNodes[i].disabled = true;
				}
				else {
					dojo.toggleClass(this.tabSection.childNodes[i], "selected", false);
					this.tabSection.childNodes[i].disabled = false;
				}
			}
			this.showLicenseDescription(activeLicense);
			this.updateLicenseButton(activeLicense);
			this.showAvailableCount(activeLicense);
			this.displayMessage("");
		}
	},
	
	getLicenseInfoErrorCallback: function (err) {
		this.displayErrorMessage(err.message);
	},
	
	runLatestSearch: function () {
		if (this.latestSearch != null) {
			this.runLicenseSearch(this.latestSearch.licenseName, this.latestSearch.pageNum);
		}
	},
	
	runLicenseSearch: function (licenseName, pageNum) {
		licenseName = licenseName || this.defaultLicenseSearch;
		this.showLicenseDescription(licenseName);
		this.updateLicenseButton(licenseName);
		if (licenseName != null) {
			this.latestSearch = {
				type: "license",
				licenseName: licenseName,
				pageNum: pageNum
			};
			this.updateHeaderLinks();
			this.latestSearchTerm = licenseName;
			this.latestSearchType = "licenseType";
			this.queryPage = pageNum || 0;
			this.updatePagerDivLinks();
			dojox.data.dom.removeChildren(this.tbody);
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.appendChild(document.createTextNode("Searching for users..."));
			tr.appendChild(td);
			this.tbody.appendChild(tr);
			var srh = new ServiceResponseHandler(this, "showUsers", "handleError");
			AdminClient.getContributors({searchTerm: licenseName, isLicenseSearch: true, pageSize: PAGE_SIZE, hideGuest: true, hideUnassigned: true, pageNum: pageNum}, srh);
		}
	},
	
	showLicenseDescription: function (licenseId) {
		dojox.data.dom.removeChildren(this.licenseDescription);
		this.licenseDescription.appendChild(document.createTextNode(this.licenseDescription[licenseId]));
	},
	
	updateLicenseButton: function (licenseId) {
		dojox.data.dom.removeChildren(this.assignButton);
		if (this.licenseMap[licenseId]) {
			this.assignButton.appendChild(document.createTextNode("Assign " + this.licenseMap[licenseId].name + " Licenses"));
		}
	},
	
	showAvailableCount: function (licenseId) {
		dojox.data.dom.removeChildren(this.licenseCountArea);
		if (this.licenseCountArea[licenseId] && this.licenseCountArea[licenseId]["total"] != null &&
				this.licenseCountArea[licenseId]["used"] != null) {
			var available = this.licenseCountArea[licenseId]["unlimited"] ? "Unlimited" : this.licenseCountArea[licenseId]["total"] - this.licenseCountArea[licenseId]["used"];
			var disabled = this.licenseCountArea[licenseId]["disabled"];
			if (disabled) {
				var disabledReason = this.licenseCountArea[licenseId]["disabledReason"]
				this.displayWarningMessage("This Client Access License is disabled: " + disabledReason);
				// set a flag if this license is disabled because it is assigned to more users than are allowed
				this.isOverAssigned = disabledReason != null && disabledReason.indexOf("CRJAZ0969I") === 0;
			}
			else {
				this.isOverAssigned = false;
				this.licenseCountArea.appendChild(document.createTextNode(available + " license(s) available"));
			}
			if (!available || disabled) {
				this.assignButton.disabled = true;
			}
			else {
				this.assignButton.disabled = false;
			}
		}
	},
	
	showUsers: function (resultSet) {
		dojox.data.dom.removeChildren(this.tbody);
		if (this.pagerDiv == null) {
			this.createPagerDiv();
		}
		if (this.licenseCountArea[this.latestSearch.licenseName]) {
			this.licenseCountArea[this.latestSearch.licenseName]["used"] = resultSet.count;
			this.showAvailableCount(this.latestSearch.licenseName);
		}
		else {
			this.licenseCountArea[this.latestSearch.licenseName] = { used: resultSet.count };
		}
		if (resultSet == null || resultSet.elements == null || resultSet.elements.length == 0) {
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.appendChild(document.createTextNode("No matching users found"));
			td.colSpan = 3;
			tr.appendChild(td);
			tr.onclick = function (e) {
				dojo.stopEvent(e);
			}
			this.tbody.appendChild(tr);
			dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
			dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
			dojox.data.dom.removeChildren(this.pagerDiv.rangeSpan);
			dojox.data.dom.removeChildren(this.pagerDiv.totalSpan);
			this.pagerDiv.rangeSpan.appendChild(document.createTextNode("0"));
			this.pagerDiv.totalSpan.appendChild(document.createTextNode("0"));
		}
		else {
			dojox.data.dom.removeChildren(this.pagerDiv.rangeSpan);
			dojox.data.dom.removeChildren(this.pagerDiv.totalSpan);
			if (resultSet.count > PAGE_SIZE) {
				var lowerBound = (this.queryPage * PAGE_SIZE) + 1;
				var upperBound = (this.queryPage + 1) * PAGE_SIZE;
				if (this.queryPage == 0) {
					dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", false);
				}
				if (upperBound >= resultSet.count) {
					upperBound = resultSet.count;
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", false);
				}
				this.pagerDiv.rangeSpan.appendChild(document.createTextNode(lowerBound + " - " + upperBound));
			}
			else {
				dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				this.pagerDiv.rangeSpan.appendChild(document.createTextNode("1 - " + resultSet.count));
			}
			this.pagerDiv.totalSpan.appendChild(document.createTextNode(resultSet.count));
			dojo.forEach(resultSet.elements, dojo.hitch(this, function(user){
				var nameNode = document.createElement("div");
				var unassignLicenseIcon = document.createElement("img");
				unassignLicenseIcon.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif");
				unassignLicenseIcon.title = "Unassign License";
				unassignLicenseIcon.className = "archiveIcon";
				unassignLicenseIcon.itemId = user.itemId;
				unassignLicenseIcon.userId = user.userId;
				
				this.connect(unassignLicenseIcon,"onmouseover", function (e) {
					e.target.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove-hov.gif");
				});
				this.connect(unassignLicenseIcon, "onmouseout", function (e) {
					e.target.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif");
				});
				this.connect(unassignLicenseIcon, "onclick", function (e) {
					dojo.stopEvent(e);
					this.unassignLicense(e.target.userId);
				});
				nameNode.appendChild(unassignLicenseIcon);
				nameNode.appendChild(document.createTextNode(user.name));
				var row = this.addRow(null, [nameNode, user.userId, user.emailAddress]);
				dojo.toggleClass(row.parentNode,"userTable",true);
			}));
		}
	},
	
	assignLicenses: function (newUsers) {
		if (newUsers != null) {
			var userIds = [];
			for (var i=0; i < newUsers.length; i++) {
				var user = newUsers[i];
				userIds.push(user.userId);
			}
			var args = {
				jsonUserIds: dojo.toJson(userIds),
				licenseId: this.latestSearch.licenseName
			};
			var srh = new ServiceResponseHandler(this, "assignLicensesCallback", "assignLicensesErrorCallback");
			LicenseClient.assignLicenseMultiple(srh, args);
		}
	},
	
	assignLicensesCallback: function (response) {
		this.runLatestSearch();
		if (response != null && response != "") {
			var errors = dojo.fromJson(response);
			var div = document.createElement("div");
			div.appendChild(document.createTextNode("An error was encountered assigning licenses to one or more users:"));
			div.appendChild(document.createElement("br"));
			for (var i=0; i < errors.length; i++) {
				div.appendChild(document.createElement("br"));
				div.appendChild(document.createTextNode(errors[i].userId + ": " + errors[i].message));
			}
			this.displayErrorMessage(div, true);
		}
		else {
			this.displayMessage("");
		}
	},
	
	assignLicensesErrorCallback: function (err) {
		this.displayErrorMessage("Error unassigning license: " + err.message);
	},
	
	unassignLicense: function (itemId) {
		var srh = new ServiceResponseHandler(this, "unassignLicenseCallback", "unassignLicenseErrorCallback");
		LicenseClient.unassignLicense(srh, { userId: itemId, licenseId: this.latestSearch.licenseName });
	},
	
	unassignLicenseCallback: function (response) {
		if (this.isOverAssigned) {
			this.reload();
		}
		else {
			this.runLatestSearch();
			this.displayMessage("");
		}
	},
	
	unassignLicenseErrorCallback: function (err) {
		this.displayErrorMessage("Error unassigning license: " + err.message);
	},
	
	createPagerDiv: function () {
		this.pagerDiv = document.createElement("div");
		dojo.toggleClass(this.pagerDiv, "pager", true);
		var prevSpan = document.createElement("span");
		var prevLink = document.createElement("a");
		prevLink.appendChild(document.createTextNode("Previous"));
		prevSpan.appendChild(prevLink);
		var disabledPrev = document.createElement("span");
		disabledPrev.appendChild(document.createTextNode("Previous"));
		dojo.toggleClass(disabledPrev, "disabledText", true);
		prevSpan.appendChild(disabledPrev);
		this.pagerDiv.appendChild(prevSpan);
		this.pagerDiv.prevSpan = prevSpan;
		this.pagerDiv.prevLink = prevLink;
		this.pagerDiv.appendChild(document.createTextNode(" | "));
		var rangeSpan = document.createElement("span");
		dojo.toggleClass(rangeSpan, "bold", true);
		this.pagerDiv.appendChild(rangeSpan);
		this.pagerDiv.rangeSpan = rangeSpan;
		this.pagerDiv.appendChild(document.createTextNode(" of "));
		var totalSpan = document.createElement("span");
		dojo.toggleClass(totalSpan, "bold", true);
		this.pagerDiv.appendChild(totalSpan);
		this.pagerDiv.totalSpan = totalSpan;
		this.pagerDiv.appendChild(document.createTextNode(" | "));
		var nextSpan = document.createElement("span");
		var nextLink = document.createElement("a");
		nextLink.appendChild(document.createTextNode("Next"));
		nextSpan.appendChild(nextLink);
		var disabledNext = document.createElement("span");
		disabledNext.appendChild(document.createTextNode("Next"));
		dojo.toggleClass(disabledNext, "disabledText", true);
		nextSpan.appendChild(disabledNext);
		this.pagerDiv.appendChild(nextSpan);
		this.pagerDiv.nextSpan = nextSpan;
		this.pagerDiv.nextLink = nextLink;
		this.tableControls.appendChild(this.pagerDiv);
		this.updatePagerDivLinks();
		this.connect(prevLink,"onclick",function (e) {
			dojo.stopEvent(e);
			document.location.href = e.target.href;
		});
		this.connect(nextLink,"onclick",function (e) {
			dojo.stopEvent(e);
			document.location.href = e.target.href;
		});
	},
	
	updatePagerDivLinks: function () {
		if (this.pagerDiv != null) {
			this.pagerDiv.prevLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage - 1);
			this.pagerDiv.nextLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage + 1);
		}
	}
});

})();